# -*- coding: utf-8 -*-
import os
from jinja2 import Environment, FileSystemLoader

from cpbox.tool import file
from cpbox.tool import utils

def render_to_str(template_file, data):
    if not os.path.isfile(template_file):
        raise Exception('template file not found: ' +  template_file)
    template_dir = os.path.dirname(template_file)
    template_name = os.path.basename(template_file)
    file_loader = FileSystemLoader(template_dir)
    env = Environment(loader=file_loader)
    template = env.get_template(template_name)
    result = template.render(data)
    return result

def render_to_file(template_file, data, target_file):
    if not os.path.isfile(template_file):
        raise Exception('template file not found: ' +  template_file)
    ret = render_to_str(template_file, data)
    file.file_put_contents(target_file, ret)
    return ret

def render(template_file, config_file=None, config_section=None, extra_kvs=None, output_file=None, output_dir=None, **kwargs):
    data = utils.load_yaml_config(config_file, config_section, extra_kvs)
    if not output_file and not output_dir:
        raise Exception('should specify output_file or output_dir')

    if output_file is not None:
        output_file = output_file
    else:
        output_file = os.path.join(output_dir.rstrip('/'), os.path.basename(template_file))
    return render_to_file(template_file, data, output_file)

def render_str(patten, data):
    env = Environment(keep_trailing_newline=True)
    patten = file.ensure_decode(patten)
    s = env.from_string(patten)
    ret = s.render(data)
    return ret



import os
import platform
import re
import subprocess
import sys
from subprocess import CalledProcessError

from cpbox.tool import datatypes


def get_os_name():
    os_name = platform.system()
    map = {
            'Linux': 'linux',
            'Darwin': 'mac',
            'CYGWIN': 'cygwin',
            'MINGW': 'MinGw',
            }
    for name, sys_name in map.items():
        if os_name.startswith(name):
            return sys_name
    return 'unknown'

def available_cpu_count():
    """ Number of available virtual or physical CPUs on this system, i.e.
    user/real as output by time(1) when called with an optimally scaling
    userspace-only program"""

    # cpuset
    # cpuset may restrict the number of *available* processors
    try:
        m = re.search(r'(?m)^Cpus_allowed:\s*(.*)$',
                      open('/proc/self/status').read())
        if m:
            res = bin(int(m.group(1).replace(',', ''), 16)).count('1')
            if res > 0:
                return res
    except IOError:
        pass

    # Python 2.6+
    try:
        import multiprocessing
        return multiprocessing.cpu_count()
    except (ImportError, NotImplementedError):
        pass

    # https://github.com/giampaolo/psutil
    try:
        import psutil
        return psutil.cpu_count()   # psutil.NUM_CPUS on old versions
    except (ImportError, AttributeError):
        pass

    # POSIX

def memtotal_bytes():
    bytes = os.sysconf('SC_PAGE_SIZE') * os.sysconf('SC_PHYS_PAGES')
    return bytes

def memtotal_mb():
    return memtotal_bytes() / (1024 ** 2)

# adapt to `commands.getstatusoutput` for python 3+
# https://stackoverflow.com/questions/11344557/replacement-for-getstatusoutput-in-python-3
def run_cmd(cmd, stdout=None):
    ret = 0
    try:
        output = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
    except CalledProcessError as e:
        ret = e.returncode
        output = e.output
    output = output.decode('utf-8')
    return ret, output.rstrip('\n')
# the difference between `subprocess.call()` and `subprocess.check_output()`
# https://stackoverflow.com/questions/25333537/performance-of-subprocess-check-output-vs-subprocess-call
def shell_run(cmd, keep_pipeline=True, exit_on_error=True, stdout=None, stderr=None):
    if datatypes.is_list_or_tuple(cmd):
        cmd = ';\n'.join(cmd)
    return_code = 0
    ret = subprocess.run(cmd, shell=True, stdout=stdout, stderr=stderr, check=False)
    return_code = ret.returncode
    if return_code != 0 and exit_on_error:
        sys.exit(return_code)
    return return_code



import socket
import os
import sys
import json

from os import path
from cpbox.tool import system
from cpbox.tool import net


def fg_mode():
    return '-it' if sys.stdout.isatty() else '-i'


def get_docker_network_gw(name):
    cmd = 'docker network inspect %s' % (name)
    code, ret = system.run_cmd(cmd)
    network_info = json.loads(ret)
    return network_info[0]['IPAM']['Config'][0]['Gateway']


def docker0_ip():
    if system.get_os_name() == 'mac':
        return get_docker_network_gw('bridge')
    else:
        return net.get_ip_address('docker0')


def base_docker_args(container_name, volumes=None, ports=None, envs=None, working_dir=None):
    args = '--name ' + container_name

    if working_dir:
        args += ' -w ' + working_dir

    if volumes:
        if isinstance(volumes, dict):
            args += ' ' + ' '.join(('-v ' + str(from_) + ':' + to_ for from_, to_ in volumes.items()))
        else:
            args += ' ' + ' '.join(('-v ' + item for item in volumes))
    if ports:
        if isinstance(ports, dict):
            args += ' ' + ' '.join(('-p ' + str(from_) + ':' + str(to_) for from_, to_ in ports.items()))
        else:
            args += ' ' + ' '.join(('-p ' + str(item) for item in ports))
    if envs:
        args += ' ' + ' '.join(('-e ' + item for item in envs))

    if path.isfile('/etc/localtime'):
        args += ' -v /etc/localtime:/etc/localtime'

    return args



import string
import random

import re
import six

first_cap_re = re.compile('(.)([A-Z][a-z]+)')
all_cap_re = re.compile('([a-z0-9])([A-Z])')


def random_str(count=10):
    return ''.join(random.choice(string.ascii_letters) for i in range(count))


def uncamelize_key(key):
    if not isinstance(key, six.string_types):
        return key
    s1 = first_cap_re.sub(r'\1_\2', key)
    return all_cap_re.sub(r'\1_\2', s1).lower()


def uncamelize_value(value):
    if isinstance(value, six.string_types):
        return value
    if isinstance(value, dict):
        return {uncamelize_key(k): uncamelize_value(v) for k, v in value.items()}
    if isinstance(value, list):
        return [uncamelize_value(v) for v in value]
    if isinstance(value, tuple):
        return tuple((uncamelize_value(v) for v in value))
    return value


def camelize(key):
    return ''.join(s[0].upper() + s[1:] for s in key.split('_'))


def camelize_key(key):
    return '.'.join(camelize(x) for x in key.split('.'))


def camelize_value(value):
    if isinstance(value, six.string_types):
        return value
    if isinstance(value, dict):
        return {camelize_key(k): camelize_value(v) for k, v in value.items()}
    if isinstance(value, list):
        return [camelize_value(v) for v in value]
    if isinstance(value, tuple):
        return tuple((camelize_value(v) for v in value))
    return value



import unittest
import inspect

from cpbox.app.devops import DevOpsApp
from cpbox.tool import functocli


class App(DevOpsApp):
    def __init__(self):
        DevOpsApp.__init__(self, 'test-app')

    def default_func(self):
        print(dir(self))


class ToolA():

    def __init__(self, app, name, port):
        self.app = app
        self.name = name
        self.port = port
        print('ToolA: %s %s' % (app, name))

    def show_name(self):
        print(self.name)

    def show_port(self):
        print(self.port)


class ToolkitBase(object):

    def __init__(self):
        pass

    def base_name(self):
        print('ToolkitBase')

    @functocli.keep_method
    def version(self):
        print('1.0.0')


@functocli.merge_class(ToolA)
class Toolkit(ToolkitBase):

    def __init__(self):
        print('__init__: %s' % (self))
        self._init_tool_a()
        self.name = 'Toolkit'

    def _init_tool_a(self):
        functocli.append_cli(self, ToolA(self, 'tool-a', port=80))

    def show_cli_name(self):
        print(self.name)


class TestFunc(unittest.TestCase):
    def test_run(self):
        pass


if __name__ == '__main__':
    # functocli.run_app(Toolkit, 'debug')
    # tool_a = ToolA(None, 'tool-a', port=80)
    # func = getattr(tool_a, 'show_name')
    # func()
    DevOpsApp.run_app(App, default_method='default_func')
    # unittest.main()



import unittest

from cpbox.tool import concurrent
from cpbox.tool import logger

class TimerTest:

    def __init__(self):
        self.count = 0
        self.total = 100

    def right(self):
        return self.count == self.total

    def run(self):

        def callback():
            self.count += 1

        timer = concurrent.TRepeatedTimer(callback, 0.001, max_count=self.total)
        timer.start()
        timer.join()

class TestTimer(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_timer(self):
        test = TimerTest()
        test.run()
        self.assertTrue(test.right())

if __name__ == '__main__':
    logger.make_logger_for_test()
    unittest.main()
